// globals.c

/*  Copyright (C) 1996-1997  Id Software, Inc.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

    See file, 'COPYING', for details.
*/

#include "qbsp.h"

// A convenient collection of most shared globals in the program

const int rgcMemSize[GLOBAL+1] = {
	sizeof(char),		sizeof(dplane_t),	sizeof(byte),		sizeof(dvertex_t),
	sizeof(byte),		sizeof(dnode_t),	sizeof(texinfo_t),	sizeof(dface_t),
	sizeof(byte),		sizeof(dclipnode_t),sizeof(dleaf_t),	sizeof(unsigned short),
	sizeof(dedge_t),	sizeof(int),		sizeof(dmodel_t),	sizeof(mapface_t),
	sizeof(mapbrush_t),	sizeof(mapentity_t),sizeof(winding_t),	sizeof(face_t),
	sizeof(plane_t),	sizeof(portal_t),	sizeof(surface_t),	sizeof(node_t),
	sizeof(brush_t),	sizeof(miptex_t),	sizeof(wvert_t),	sizeof(wedge_t),
	sizeof(hashvert_t),	1,					1};

mapdata_t map;
int hullnum;

// Useful shortcuts
mapentity_t *pCurEnt;
mapentity_t *pWorldEnt;

// Move these into mapdata_t?
miptex_t *rgszMiptex;
plane_t *pPlanes;
int cMiptex;
int cPlanes;

// Mathlib.c
vec3_t vec3_origin = {0,0,0};

// util.c
File LogFile;

char *rgszWarnings[cWarnings] = {
	"No wadfiles specified in worldmodel",
	"No valid wadfiles in worldmodel",
	"Line %d: Multiple info_player_start entities",
	"Line %d: Brush with duplicate plane",
	"Line %d: Brush plane with no normal",
	"No info_player_start entity in level",
	"No info_player_deathmatch entities in level",
	"No info_player_coop entities in level",
	"Healing point %s off plane by %.3g",
	"Line %d: Couldn't create brush with %d faces, %s",

	"Reached occupant at %s, %s",				// 10
	"Bsp has %i models, map has %i in UpdateEntLump",
	"New portal was clipped away in CutNodePortals_r near %s",
	"Line %d: Brush not closed",
	"Vis leafs %d exceed normal engine max %d",
	"%s isn't a wadfile",
	"Texture %s not found",
	"%s is an invalid option",
	"Unable to open .qbsp.log",	// gb
	"No entities in empty space -- no filling performed",

	"Strange map face count",				// 20
	"Too many edges in TryMerge",
	"Healing degenerate edge at %s",
	"Line %d: Bad target%s for rotation entity",
	"Degenerate edge at %s",
	"Line %d: Degenerate QuArK-style brush texture",
	"Face with too few (%d) points in CheckFace at %s",
	"Mixed face contents (%s, %s) near %s",
	"Couldn't find non-perpendicular texture axis",
	"Line %d: Too few (%d) faces in brush",

	"Clipnodes %d exceed normal engine max %d",		// 30
	"Marksurfaces %d exceed normal engine max %d",
	"Models %d exceed normal engine max %d",
	"Planes %d exceed normal engine max %d",
	"Nodes %d exceed normal engine max %d",
	"Leafs %d exceed normal engine max %d",
	"Vertexes %d exceed normal engine max %d",
	"Faces %d exceed normal engine max %d",
	"Texinfo %d exceed normal engine max %d",
	"Line %d: Too many (%d) faces in brush",
};

char *rgszErrors[cErrors] = {
	"No leak node in WriteLeakNode",
	"Unknown option '%s'",
	"Line %d: Entity key or value too long",
	"Line %d: Invalid brush plane format",
	"Line %d: Internal error: map.iFaces >= map.cFaces",
	"Line %d: Invalid entity format, { not found",
	"Internal error: map.iEntities >= map.cEntities",
	"Unexpected EOF (no closing brace)",
	"Internal error: cMiptex > map.cFaces",
	"Internal error: pWorldEnt->iTexinfo > pWorldEnt->cTexinfo",

	"%s is version %i, not %i",					// 10
	"No axis found for winding",
	"Points on winding exceeded estimate",
	"Line %d: Entity with no valid brushes or textures",
	"No valid planes in surface list",
	"Surface with no faces",
	"Bad contents in face",
	"Mixed face contents (%s, %s) near %s",
	"Surface without a face",
	"Line %d: Texture name \"%s\" too long",

	"Found a non-convex face at %s",					// 20
	"Found a non-canonical vector",
	"Internal error: numbrushplanes >= cPlanes",
	"Normalization error in FindPlane (vector length %.4f)",
	"Internal error: f->numpoints > MAXEDGES",
	"Invalid normal (vector length %.4f)",
	"Line %d: Internal error: numbrushfaces == MAX_FACES",
	"Internal error: num_hull_points == MAX_HULL_POINTS",
	"Internal error: num_hull_edges == MAX_HULL_EDGES",
	"Attempting to split freed face",

	"Internal error: numpoints > MAXEDGES in SplitFace",		// 30
	"Valid hulls are 0-2",
	"Face coordinate out of range %s",
	"Deformed lump in BSP file (%d size is not divisible by %d)",
	"Failed to open %s: %s",
	"Failure reading from file %s",
	"Failure writing to file %s",
	"Failure seeking to %d in file %s",
	"Portal already included in AddPortalToNodes near %s",
	"Portal not in leaf near %s",

	"Portal not bounding leaf near %s",				// 40
	"Mislinked portal in CutNodePortals_r near %s",
	"Didn't split the polygon in SubdivideFace near %s, %s",
	"Internal error: map.rgEntities[map.iEntities].iVertices > map.rgEntities[map.iEntities].cVertices",
	"0 contents in GetEdge",
	"Internal error: map.rgEntities[map.iEntities].iEdges >= map.rgEntities[map.iEntities].cEdges",
	"Internal error: map.rgEntities[0].iPlanes >= map.rgEntities[0].cPlanes",
	"Internal error: numwedges >= cWEdges",
	"Internal error: numwverts >= cWVerts",
	"f->original exists in SplitFaceForTjunc",

	"Internal error: Invalid memory %i type in AllocMem",		// 50
	"Too many points on winding (%d, max = %d)",
	"Out of memory",
	"Internal error: data + len - map.rgEntities[0].pTexdata > map.rgEntities[0].cTexdata",
	"Line %d: Line is incomplete",
	"Line %d: EOF inside quoted token",
	"Line %d: Token too large",
	"Invalid argument to option %s",
	"Internal error: numleaks > num_visportals",
	"Nodes in map exceed %d",

	"Source file %s may not have bsp extension",			// 60
	"Leak distance must be > 0",
	"Simplification distance must be > 1",
	"No entities in map",
	"Line %d: No classname in entity",
	"Line %d: No world brushes",
	"Line %d: Multiple world entities",
	"Hull expansion must be >= 0",
	"Priority must be 0-2",
	"Line %d: World is not first entity",
	
	"Wad wildcard path exceeded (max = %d kb)",			// 70
	"Leafs in map exceed %d",
	"Planes in map exceed %d",
	"Vertexes in map exceed %d",
	"Faces in map exceed %d",
	"Marksurfaces in map exceed %d",
	"Texinfo in map exceed %d",
	"Epsilon must be > 0",
	"Line %d: Internal error: map.iBrushes >= map.cBrushes",
};
